library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use std.textio.all;
use ieee.math_real.all;

entity q_tb is
end;

architecture arch of q_tb is
  
  -- wav audio file
  constant wav_file : string :="test6.wav";
  
  -- signals of dut
  signal clk,clk_en,test : std_logic:='0';
  signal x     : std_logic_vector(7  downto 0);
  signal X_dft : std_logic_vector(31 downto 0);

  -- parameters of simulation
  constant clk_frequency : real := 2.048e6;
  constant sT : time := (0.5/clk_frequency) * 1 sec;-- semi-period
  constant n_ce: natural := 256;-- T_clk_en=T_clk*n_ce
  constant tms : real := 1.0;-- time of simulation in seconds
  constant n_clk_sT : natural := natural(2.0*tms*clk_frequency);--number of clk semi-periods

  -- file section
  type t_char_file is file of character;
  type t_int_file is file of integer;
  
  file f0 : t_char_file;-- *.wav input; little-endian, 16bit, mono, 48kHz
  --file f1 : text;-- capa 1 output
  --file f2 : text;-- capa 2 output
  --file f3 : text;-- capa 3 output
  --file input is the "STD_INPUT" (terminal)
  --file output is the "STD_OUTPUT" (terminal)
  
begin
  
  dut: entity work.BP_v1(rtl_opt)
    port map (clk=>clk,clk_en=>clk_en,test=>test,
              X_dft=>X_dft,
              x=>x);
  
------------------------------------------------------
  
  clk_process : process 
  begin
    clk <= '0';wait for sT;
    for i in 1 to n_clk_sT loop    
      clk <= not clk;
      wait for sT;
    end loop;
    wait;
  end process;

  ------------------------------------------------------
  
  clk_en_process : process(clk)
    variable n_ce_counter : natural := n_ce-1;
  begin
    if rising_edge(clk) then
      clk_en<='0';
      if n_ce_counter = n_ce-1 then
        n_ce_counter:=0;
        clk_en<='1';
      else
        n_ce_counter:=n_ce_counter+1;
      end if;
    end if;
  end process;

  ------------------------------------------------------
  
  x_process: process
    variable l0,l1,l2,l3: line;
    variable l0_cp,l1_cp,l2_cp,l3_cp: line;-- copy to debug using std_output
    variable c0,c1,c2,c3: character;
    variable long: natural;
    variable n1, n2, i : natural;
    variable int8: integer;
    variable nat8: natural;
    variable x_var: std_logic_vector(7 downto 0);
    
  begin
    file_open(f0, wav_file, read_mode);
    --file_open(f1, "capa1", write_mode);

    -- length of f0 file
    long:=0;
    while not endfile(f0) loop
      read(f0,c0);
      long:=long+1;
    end loop;
    file_close(f0);
    
    file_open(f0, wav_file, read_mode);

    -- clean header
    n1 := 44;if n1>long then n1:=long; end if;
    for i in 1 to n1 loop
      read(f0,c0);
    end loop;
    
    -- read 16-bit-sample and convert to 8-bit-sample
    -- it seems PCM 16-bit uses signed type
    -- so,
    -- 1000 0000 0000 0000b is the minimum value 
    -- 0111 1111 1111 1111b is the maximum value
    -- firstly we will truncate
    -- as little endian is used we only take the second byte (the MSbyte)
    -- 1000 0000 will be the minimum value (-128d)
    -- 0111 1111 will be the maximum value (+127d)
    -- secondly we convert to unsigned adding +128
    -- samples from the file are sampled at 48kHz
    -- we want samples at 8kHz
    -- so only 1 out of 6 samples are needed
    -- and of this samples we only take half of the bits
    -- so 1 out of 12 bytes are needed

    -- read some data
    while not endfile(f0) loop
    --n2 := 12*300-9600+9600+44;if n2>long then n2:=long; end if;i:=44;while (i < n2) loop
      for k in 1 to 12 loop
        read(f0,c0);
      end loop;
      int8:=character'pos(c0);-- read as unsigned!!!
      if int8>=128 then int8:=int8-256;end if;--unsigned to signed
      nat8:=int8+128;--unsigned

      wait until rising_edge(clk_en);
      x_var:=std_logic_vector(to_unsigned(nat8,8));--debug
      x<=std_logic_vector(to_unsigned(nat8,8));

      
      --write(l0,nat8);writeline(output,l0);-- to debug
      i := i + 12;
    end loop;
    
    file_close(f0);
    
    wait;
  end process;
  
  ------------------------------------------------------
  
  debug_process: process(test)
    variable l0: line;
    variable c: character :='0';
  begin
    if test='1' then c:='1';elsif test='0' then c:='0';else c:=' ';end if;
    write(l0,c);writeline(output,l0);-- to debug
  end process;
  
end;

